@@ -82,17 +82,6 @@
 #include "mb/pg_wchar.h"
 #endif
 
-#ifdef HAVE_SIGPROCMASK
-sigset_t	UnBlockSig,
-			BlockSig,
-			AuthBlockSig;
-
-#else
-int			UnBlockSig,
-			BlockSig,
-			AuthBlockSig;
-#endif
-
 #define RECOVERY_LOOP_END	(0)
 #define RECOVERY_LOOP_CONTINUE	(1)
 #define RECOVERY_LOOP_FAIL	(2)
@@ -130,83 +119,36 @@ read_packet(int sock,RecoveryPacket * packet)
 	char * read_ptr;
 	int read_size = 0;
 	int packet_size = 0;
-	int cnt = 0;
 
 	read_ptr = (char*)packet;
 	packet_size = sizeof(RecoveryPacket);
+
 	for (;;){
 		r = recv(sock,read_ptr + read_size ,packet_size, MSG_WAITALL);
-		if (r < 0){
-			if (errno == EINTR)
-			{
+		if (r < 0) {
+			if (errno == EINTR || errno == EAGAIN) {
 				continue;
+			} else {
+				elog(DEBUG1, "read_packet():recv failed");
+				return -1;
 			}
-#ifdef EAGAIN
-			else if (errno == EAGAIN)
-			{
-				usleep(PGR_RECV_WAIT_MSEC);
-				continue;
-			}
-#endif /* EAGAIN */
-#ifdef ECONNREFUSED
-			else if (errno == ECONNREFUSED)
-			{
-				usleep(PGR_RECV_WAIT_MSEC);
-				continue;
-			}
-#endif /* ECONNREFUSED */
-#ifdef ENOTCONN
-			else if (errno == ENOTCONN)
-			{
-				usleep(PGR_RECV_WAIT_MSEC);
-				continue;
-			}
-#endif /* ENOTCONN */
-			if (cnt < PGR_RECV_RETRY_CNT )
-			{
-				cnt ++;
-				usleep(PGR_RECV_WAIT_MSEC);
-				continue;
-			}
-			else
-			{
-				if (read_size == 0)
-				{
-					return -1;
-				}
-			}
+		} else if (r == 0) {
+			elog(DEBUG1, "read_packet():unexpected EOF");
 			return -1;
-		}
-		if (r > 0)
-		{
+		} else /*if (r > 0)*/ {
 			read_size += r;
-			if (read_size == packet_size)
-			{
+			if (read_size == packet_size) {
 				show_recovery_packet(packet);
 				return read_size;
 			}
 		}
-		if ((r == 0) && (read_size == 0))
-		{
-			return -1;
-		}
-		if (cnt < PGR_RECV_RETRY_CNT )
-		{
-			cnt ++;
-			usleep(PGR_RECV_WAIT_MSEC);
-			continue;
-		}
-		else
-		{
-			return -1;
-		}
 	}
+	return -1;
 }
 
 static int
 send_recovery_packet(int  sock, RecoveryPacket * packet)
 {
-
 	char * send_ptr;
 	int send_size= 0;
 	int buf_size = 0;
@@ -221,30 +163,53 @@ send_recovery_packet(int  sock, RecoveryPacket * packet)
 	/*
 	 * Wait for something to happen.
 	 */
-	FD_ZERO(&wmask);
-	FD_SET(sock,&wmask);
-	rtn = select(sock+1, (fd_set *)NULL, &wmask, (fd_set *)NULL, &timeout);
-	if (rtn && FD_ISSET(sock, &wmask))
+	rtn = 1;
+	while (rtn)
 	{
-		send_ptr = (char *)packet;
-		buf_size = sizeof(RecoveryPacket);
-
 		for (;;)
 		{
-			s = send(sock,send_ptr + send_size,buf_size - send_size ,0);
-			if (s < 0){
-				if (errno == EINTR)
+			timeout.tv_sec = PGR_SEND_TIMEOUT;
+			timeout.tv_usec = 0;
+
+			FD_ZERO(&wmask);
+			FD_SET(sock,&wmask);
+			rtn = select(sock+1, (fd_set *)NULL, &wmask, (fd_set *)NULL, &timeout);
+
+			if (rtn < 0)
+			{
+				if (errno == EINTR || errno == EAGAIN)
 				{
 					continue;
 				}
-				return STATUS_ERROR;
-			}
-			if (s > 0)
-			{
-				send_size += s;
-				if (send_size == buf_size)
+				else
 				{
-					return STATUS_OK;
+					rtn = 0;
+					break;
+				}
+			}
+			else if (rtn && FD_ISSET(sock, &wmask))
+			{
+				send_ptr = (char *)packet;
+				buf_size = sizeof(RecoveryPacket);
+
+				s = send(sock,send_ptr + send_size,buf_size - send_size ,0);
+				if (s < 0) {
+					if (errno == EINTR || errno == EAGAIN) {
+						continue;
+					}
+					elog(DEBUG1, "send_recovery_packet():send error");
+
+					/* EPIPE || ENCONNREFUSED || ENSOCK || EHOSTUNREACH */
+					return STATUS_ERROR;
+				} else if (s == 0) {
+					elog(DEBUG1, "send_recovery_packet():unexpected EOF");
+					return STATUS_ERROR;
+				} else /*if (s > 0)*/ {
+					send_size += s;
+					if (send_size == buf_size)
+					{
+						return STATUS_OK;
+					}
 				}
 			}
 		}
@@ -255,7 +220,6 @@ send_recovery_packet(int  sock, RecoveryPacket * packet)
 static int
 send_packet(int * sock, RecoveryPacket * packet )
 {
-
 	int count = 0;
 	ReplicateServerInfo * host = NULL;
 
@@ -289,7 +253,6 @@ send_packet(int * sock, RecoveryPacket * packet )
 static void
 master_loop(int fd)
 {
-
 	int count;
 	int sock;
 	int status = STATUS_OK;
@@ -326,10 +289,15 @@ master_loop(int fd)
 		if (rtn && FD_ISSET(sock, &rmask))
 		{
 			r_size = read_packet(sock,&packet);
-			if (r_size <= 0)
+			if (r_size == 0)
 			{
 				continue;
 			}
+			else if (r_size < 0)
+			{
+			    loop_end=true;
+			    break;
+		        }
 		}
 		else
 		{
@@ -359,18 +327,37 @@ master_loop(int fd)
 				PGR_Set_Cluster_Status(STATUS_RECOVERY);
 				loop_end = true;
 				break;
+			case RECOVERY_ERROR_TARGET_ONLY:	
+				memset(&packet,0,sizeof(packet));
+				set_recovery_packet(&packet, RECOVERY_ERROR_ANS );
+				status = send_packet(&sock,&packet);
+				PGR_Set_Cluster_Status(STATUS_REPLICATE);
+				break;
 			case RECOVERY_ERROR_CONNECTION:
 				memset(&packet,0,sizeof(packet));
 				set_recovery_packet(&packet, RECOVERY_ERROR_ANS );
 				status = send_packet(&sock,&packet);
 				PGR_Set_Cluster_Status(STATUS_REPLICATE);
+				/**
+				 * kill broken cluster db.
+				 * FIXME: missing MyProcPid here. It must be postmaster's pid.
+				 * but here's a bug MyProcPid doesn't initialized properly , so MyProcPid = postmaster's pid.				
+				 * To fix this, define variable to set posmaster's pid.
+				 */
+				kill(MyProcPid,SIGQUIT);
 				loop_end = true;
 				break;
 			case RECOVERY_ERROR_ANS:
+			  /* TODO: recovery failed. close this postmaster */
+			        loop_end = true;
+			        break;
 			case RECOVERY_FINISH:
 				PGR_Set_Cluster_Status(STATUS_REPLICATE);
 				loop_end = true;
 				break;
+             		default:
+		                loop_end = true;
+				break;
 		}
 		if (loop_end)
 		{
@@ -397,9 +384,10 @@ PGR_Master_Main(void)
 	gethostname(Local_Host_Name,sizeof(Local_Host_Name));
 	pqsignal(SIGHUP, authdie);
 	pqsignal(SIGTERM, authdie);
+	pqsignal(SIGINT, authdie);
 	pqsignal(SIGQUIT, authdie);
 	pqsignal(SIGALRM, authdie);
-	PG_SETMASK(&AuthBlockSig);
+	PG_SETMASK(&UnBlockSig);
 
 	status = STATUS_ERROR;
 	status = PGR_Create_Socket_Bind(&fd, "", RecoveryPortNumber);
@@ -511,18 +499,19 @@ rsync_pg_data(char * src, char * dest)
 {
 	int status;
 	char *args[12];
-	int pid;
+	int pid, i = 0;
 
-	args[0] = "rsync";
-	args[1] = "-a";
-	args[2] = "-r";
-	args[3] = "-z";
-	args[4] = "--delete";
-	args[5] = "-e";
-	args[6] = RsyncOption;
-	args[7] = src;
-	args[8] = dest;
-	args[9] = NULL;
+	args[i++] = "rsync";
+	args[i++] = "-a";
+	args[i++] = "-r";
+	if (RsyncCompress)
+		args[i++] = "-z";
+	args[i++] = "--delete";
+	args[i++] = "-e";
+	args[i++] = RsyncOption;
+	args[i++] = src;
+	args[i++] = dest;
+	args[i++] = NULL;
 
 	pid = fork();
 	if (pid == 0)
@@ -531,10 +520,21 @@ rsync_pg_data(char * src, char * dest)
 	}
 	else
 	{
-		wait(&status);
-		if (status < 0)
+		for (;;)
 		{
-			return STATUS_ERROR;
+			int result;
+			result = wait(&status);
+			if (result < 0)
+			{
+				if (errno == EINTR)
+					continue;
+				return STATUS_ERROR;
+			}
+
+			if (WIFEXITED(status) == 0 || WEXITSTATUS(status) != 0)
+				return STATUS_ERROR;
+			else
+				break;
 		}
 	}
 	return STATUS_OK;
@@ -716,46 +716,72 @@ recovery_rsync(char * src , char * dest, int stage)
 	}
 
 	/* recovery step of "global" directory */
+	fprintf(stderr,"%s recovery step of [global] directory...",
+			((stage == 1)?"1st":"2nd"));
 	if (rsync_global_dir(src, dest) != STATUS_OK)
 	{
+		fprintf(stderr,"NG\n");
 		return STATUS_ERROR;
 	}
+	fprintf(stderr,"OK\n");
 
 	if (stage == PGR_1ST_RECOVERY)
 	{
 		/* 1st recovery step of "base" directory */
+		fprintf(stderr,"1st recovery step of [base] directory...");
 		if (first_recovery(src,dest,"base") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
+
+		fprintf(stderr,"1st recovery step of [pg_clog] directory...");
 		/* 1st recovery step of "pg_clog" directory */
 		if (first_recovery(src,dest,"pg_clog") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
+
 		/* 1st recovery step of "pg_xlog" directory */
+		fprintf(stderr,"1st recovery step of [pg_xlog] directory...");
 		if (first_recovery(src,dest,"pg_xlog") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
 	}
 	else
 	{
 		/* 2nd recovery step of "base" directory */
+		fprintf(stderr,"2nd recovery step of [base] directory...");
 		if (second_recovery(src,dest,"base") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
+
 		/* 2nd recovery step of "pg_clog" directory */
+		fprintf(stderr,"2nd recovery step of [pg_clog] directory...");
 		if (second_recovery(src,dest,"pg_clog") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
+
 		/* 2nd recovery step of "pg_xlog" directory */
+		fprintf(stderr,"2nd recovery step of [pg_xlog] directory...");
 		if (second_recovery(src,dest,"pg_xlog") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
 	}
 
 	return STATUS_OK;
@@ -845,6 +871,7 @@ recovery_loop(int fd)
 			break;
 		case RECOVERY_ERROR_OCCUPIED:
 		case RECOVERY_ERROR_CONNECTION:
+	default:
 			rtn = RECOVERY_LOOP_FAIL;
 			break;
 	}
@@ -909,12 +936,12 @@ Retry_Start_Recovery:
 			{
 				continue;
 			}
-			if (status == RECOVERY_LOOP_END)
+			else if (status == RECOVERY_LOOP_END)
 			{
 				close(fd);
 				break;
 			}
-			if (status == RECOVERY_LOOP_FAIL)
+			else if (status == RECOVERY_LOOP_FAIL)
 			{
 				status = stop_recovery_send(&fd,host);
 				if (status != STATUS_OK)
@@ -925,6 +952,11 @@ Retry_Start_Recovery:
 				close(fd);
 				return STATUS_ERROR;
 			}
+			else 
+			  {
+			    close(fd);
+			    return STATUS_ERROR;
+			  }
 		}
 	}
 	return STATUS_OK;
