@@ -2808,11 +2808,7 @@ PGR_Is_Skip_Replication(char * query)
 	{
 		return 3;
 	}
-	if ((memcmp(query,SKIP_QUERY_3,strlen(SKIP_QUERY_3)) == 0) ||
-		(memcmp(query,SKIP_QUERY_4,strlen(SKIP_QUERY_4)) == 0))
-	{
-		return 1;
-	}
+
 	return 0;
 }
 
