@@ -2340,21 +2340,6 @@ PostgresMain(int argc, char *argv[], const char *username)
 				 */
 			case 'X':
 			case EOF:
-#ifdef USE_REPLICATION
-				if (Transaction_Mode == true)
-				{
-					if (PGR_Did_Commit_Transaction() == true)
-					{
-						parser_input->data = strdup("commit");
-						pg_exec_query_string(parser_input,
-											 whereToSendOutput,
-											 QueryContext);
-					}
-				}
-				/*
-				  PGR_Notice_Transaction_Query_Aborted();
-				*/
-#endif /* USE_REPLICATION */
 				/*
 				 * Reset whereToSendOutput to prevent elog from attempting
 				 * to send any more messages to client.
