@@ -35,6 +35,7 @@
 #define STATUS_CONTINUE			(4)
 #define STATUS_CONTINUE_SELECT	(5)
 #define STATUS_NOT_REPLICATE	(6)
+#define STATUS_SKIP_QUERY		(7)
 #define STATUS_REPLICATE		(10)
 #define STATUS_RECOVERY			(11)
 #define STATUS_REPLICATION_ABORT	(98)
@@ -50,12 +51,12 @@
 #define SKIP_QUERY_3 "SET autocommit TO 'on'"
 #define SKIP_QUERY_4 "SET search_path = public"
 
-#define PGR_SEND_RETRY_CNT (1500)
+#define PGR_SEND_RETRY_CNT (3)
 #define PGR_SEND_WAIT_MSEC (150)
-#define PGR_RECV_RETRY_CNT (1500)
+#define PGR_RECV_RETRY_CNT (3)
 #define PGR_RECV_WAIT_MSEC (150)
-#define PGR_SEND_TIMEOUT	(600)
-#define PGR_RECV_TIMEOUT	(30)
+#define PGR_SEND_TIMEOUT	(5)
+#define PGR_RECV_TIMEOUT	(5)
 #define PGR_LOCK_RETRY_CNT (50)
 #define PGR_LOCK_WAIT_MSEC (20)
 
@@ -91,6 +92,14 @@ typedef struct
 	char useFlag;
 } PGR_Retry_Query_Type;
 
+
+/* replicaition log */
+typedef struct {
+	uint32_t PGR_Replicate_ID;
+	uint32_t PGR_Request_ID;
+} PGR_ReplicationLog_Info;
+
+
 extern char * Query_String;
 extern int TransactionQuery;
 extern bool Transaction_Mode;
@@ -99,6 +108,7 @@ extern bool Create_Temp_Table_Mode;
 extern int RecoveryPortNumber;
 extern char * RsyncPath;
 extern char * RsyncOption;
+extern bool RsyncCompress;
 extern int TransactionSock;
 extern ReplicateNow * ReplicateCurrentTime;
 extern CopyData * PGRCopyData;
@@ -113,11 +123,15 @@ extern int PGR_Sock_To_Replication_Server;
 extern bool PGR_Need_Notice;
 extern bool PGR_Lock_Noticed;
 extern ReplicateServerInfo * CurrentReplicateServer;
+extern ReplicateServerInfo * LastReplicateServer;
 extern char * PGRSelfHostName;
 extern int PGR_Pending_Sem_Num;
 extern int PGR_Response_Mode;
 extern bool PGR_Reliable_Mode_Wait;
 extern PGR_Retry_Query_Type PGR_Retry_Query;
+extern bool PGRforceLoadBalance;
+extern bool needToUpdateReplicateIdOnNextQueryIsDone;
+extern PGR_ReplicationLog_Info ReplicationLog_Info;
 
 /* in backend/libpq/replicate.c */
 extern int PGR_Init_Replicate_Server_Data(void);
@@ -128,7 +142,7 @@ extern bool PGR_Is_Replicated_Command(char * query);
 extern int Xlog_Check_Replicate(int operation);
 extern int PGR_Replicate_Function_Call(void);
 extern void PGR_delete_shm(void);
-extern int PGR_replication(char * query_string, CommandDest dest, MemoryContext parse_context, const char * commandTag);
+extern int PGR_replication(char * query_string, CommandDest dest, Node *parsetree, const char * commandTag);
 extern bool PGR_Is_System_Command(char * query);
 extern int PGR_Call_System_Command(char * command);
 extern int PGR_GetTimeOfDay(struct timeval *tp,struct timezone *tpz);
@@ -138,6 +152,7 @@ extern int PGR_Send_Copy(CopyData * copy, int end);
 extern CopyData * PGR_Set_Copy_Data(CopyData * copy, char *str, int len, int end);
 extern char * PGR_scan_terminate( char * str);
 extern bool PGR_Is_Stand_Alone(void);
+extern void PGR_Send_Message_To_Frontend(char * msg);
 extern void PGR_Notice_Transaction_Query_Done(void);
 extern void PGR_Notice_Transaction_Query_Aborted(void);
 extern int PGR_Notice_Conflict(void);
@@ -152,6 +167,7 @@ extern void PGR_Force_Replicate_Query(void);
 extern void PGR_Notice_DeadLock(void);
 extern void PGR_Set_Cluster_Status(int status);
 extern int PGR_Get_Cluster_Status(void);
+extern int PGR_Check_Replicate_Server_Status(ReplicateServerInfo * sp);
 
 /* in backend/libpq/recovery.c */
 extern int PGR_Master_Main(void);
