@@ -5,7 +5,7 @@
  * NOTE:
  *     This file is composed of the main function of pglb.
  *
- * Portions Copyright (c) 2003, Atsushi Mitani
+ * Portions Copyright (c) 2003-2006, Atsushi Mitani
  *--------------------------------------------------------------------
  */
 /*
@@ -415,8 +415,8 @@ pglb_exit(int signal_args)
 	int rtn;
 	
 	Child_Tbl->useFlag = TBL_END;
-	signal(SIGCHLD,SIG_IGN);
-	signal(signal_args,SIG_IGN);
+	PGRsignal(SIGCHLD,SIG_IGN);
+	PGRsignal(signal_args,SIG_IGN);
 	kill (0,signal_args);
 	while (wait(NULL) > 0 )
 		;
@@ -504,7 +504,7 @@ load_balance_main(void)
 	}
 	if (Use_Connection_Pool)
 	{
-		signal(SIGCHLD,PGRrecreate_child);
+		PGRsignal(SIGCHLD,PGRrecreate_child);
 		rtn = PGRpre_fork_children(Cluster_Tbl);
 		if (rtn != STATUS_OK)
 		{
@@ -815,7 +815,7 @@ close_child(int signal_args)
 		}
 		child++;
 	}
-	signal(SIGUSR2, close_child);
+	PGRsignal(SIGUSR2, close_child);
 }
 
 /*--------------------------------------------------------------------
@@ -930,17 +930,17 @@ main(int argc, char ** argv)
 	char * w_path = NULL;
 	int detach = 1;
 
-	signal(SIGHUP, pglb_exit);
-	signal(SIGINT, pglb_exit);	
-	signal(SIGQUIT, pglb_exit);
-	signal(SIGTERM, pglb_exit);
-	signal(SIGALRM, SIG_IGN); /* ignored */
-	signal(SIGPIPE, SIG_IGN); /* ignored */
-	signal(SIGTTIN, SIG_IGN); /* ignored */
-	signal(SIGTTOU, SIG_IGN); /* ignored */
-	signal(SIGCHLD,PGRchild_wait);
-	signal(SIGUSR1, SIG_IGN); /* ignored */
-	signal(SIGUSR2, close_child); /* close child process */
+	PGRsignal(SIGHUP, pglb_exit);
+	PGRsignal(SIGINT, pglb_exit);	
+	PGRsignal(SIGQUIT, pglb_exit);
+	PGRsignal(SIGTERM, pglb_exit);
+	PGRsignal(SIGALRM, SIG_IGN); /* ignored */
+	PGRsignal(SIGPIPE, SIG_IGN); /* ignored */
+	PGRsignal(SIGTTIN, SIG_IGN); /* ignored */
+	PGRsignal(SIGTTOU, SIG_IGN); /* ignored */
+	PGRsignal(SIGCHLD,PGRchild_wait);
+	PGRsignal(SIGUSR1, SIG_IGN); /* ignored */
+	PGRsignal(SIGUSR2, close_child); /* close child process */
 	r_path = getenv("PGDATA");
 	if (r_path == NULL)
 		r_path = ".";
