@@ -6,7 +6,7 @@
  *     This file is composed of the functions to call with the source
  *     at pgreplicate for backup and cascade .
  *
- * Portions Copyright (c) 2003, Atsushi Mitani
+ * Portions Copyright (c) 2003-2006, Atsushi Mitani
  *--------------------------------------------------------------------
  */
 #ifdef USE_REPLICATION
@@ -56,21 +56,6 @@
 #include "replicate_com.h"
 #include "pgreplicate.h"
 
-void PGRinit_cascade_child(void);
-int PGRstartup_cascade(void);
-int PGRsend_lower_cascade(ReplicateHeader * header, char * query);
-int PGRsend_upper_cascade(ReplicateHeader * header, char * query);
-int PGRwait_answer_cascade(int  sock);
-ReplicateServerInfo * PGRget_lower_cascade(void);
-ReplicateServerInfo * PGRget_upper_cascade(void);
-void PGRset_cascade_server_status(ReplicateServerInfo * cascade, int status);
-ReplicateServerInfo * PGRrecv_cascade_answer(ReplicateServerInfo * cascade,ReplicateHeader * header);
-int PGRsend_cascade(int sock , ReplicateHeader * header, char * query);
-int PGRcascade_main(int sock, ReplicateHeader * header, char * query);
-int PGRwait_notice_rlog_done(void);
-int PGRsend_notice_rlog_done(int sock);
-int PGRsend_notice_quit(void);
-
 #if 0
 static int count_cascade(int flag);
 #endif
@@ -82,6 +67,7 @@ static int update_cascade_data(ReplicateHeader * header, ReplicateServerInfo * u
 static void write_cascade_status_file(ReplicateServerInfo * cascade);
 static int notice_cascade_data(int sock);
 static int notice_cascade_data_to_cluster_db(void);
+static void PGRinit_cascade_child(void); 
 
 /**
  * socket variables, moved from  Cascade_Inf->(lower|upper)->sock.
@@ -89,8 +75,8 @@ static int notice_cascade_data_to_cluster_db(void);
  * 05/10/05 tanida@sraoss.co.jp
  */
 
-int lsock=-1; /* socket for lower-cascade. */
-int usock=-1; /* socket for upper-cascade. */
+static int lsock=-1; /* socket for lower-cascade. */
+static int usock=-1; /* socket for upper-cascade. */
 
 /*--------------------------------------
  * PROTOTYPE DECLARATION
@@ -148,7 +134,7 @@ count_cascade(int flag)
 }
 #endif /* if 0 */
 
-void 
+static void 
 PGRinit_cascade_child(void) {
       fixup_socket_for_cascades(&usock,NULL);
       fixup_socket_for_cascades(&lsock,NULL);
