@@ -6,7 +6,7 @@
  * NOTE:
  *    This is the main module of the replication server.
  *
- * Portions Copyright (c) 2003, Atsushi Mitani
+ * Portions Copyright (c) 2003-2006, Atsushi Mitani
  *--------------------------------------------------------------------
  */
 #include "postgres.h"
@@ -127,7 +127,7 @@ bool PGR_Use_Replication_Log = false;
 bool	PGR_AutoCommit = true;
 unsigned int * PGR_Send_Query_ID = NULL;
 unsigned int PGR_Query_ID = 0;
-bool exit_processing = false;
+volatile bool exit_processing = false;
 int pgreplicate_pid = 0;
 
 int ReplicateSock = -1;
@@ -147,7 +147,7 @@ char * PGRuserName = NULL;
 
 int fork_wait_time = 0;
 int Idle_Flag = IDLE_MODE;
-bool Exit_Request = false;
+volatile bool Exit_Request = false;
 
 pthread_mutex_t transaction_table_mutex;
 
@@ -224,12 +224,12 @@ replicate_loop(int fd)
 
 		close(fd);
 
-		signal(SIGHUP, quick_exit);	
-		signal(SIGINT, quick_exit);	
-		signal(SIGQUIT, quick_exit);	
-		signal(SIGTERM, quick_exit);	
-		signal(SIGALRM, quick_exit); 
-		signal(SIGPIPE, SIG_IGN); 
+		PGRsignal(SIGHUP, quick_exit);	
+		PGRsignal(SIGINT, quick_exit);	
+		PGRsignal(SIGQUIT, quick_exit);	
+		PGRsignal(SIGTERM, quick_exit);	
+		PGRsignal(SIGALRM, quick_exit); 
+		PGRsignal(SIGPIPE, SIG_IGN); 
 		setpgid(0,pgid);
 		
 		if (PGRinit_transaction_table() != STATUS_OK)
@@ -245,10 +245,6 @@ replicate_loop(int fd)
 		for (;;)
 		{
 			fd_set	  rmask;
-			struct timeval timeout;
-
-			timeout.tv_sec = PGR_RECV_TIMEOUT;
-			timeout.tv_usec = 0;
 			
 			if (query != NULL)
 			{
@@ -260,7 +256,7 @@ replicate_loop(int fd)
 			 */
 			FD_ZERO(&rmask);
 			FD_SET(sock,&rmask);
-			rtn = select(sock+1, &rmask, (fd_set *)NULL, (fd_set *)NULL, &timeout);
+			rtn = select(sock+1, &rmask, (fd_set *)NULL, (fd_set *)NULL, NULL);
 			if (rtn < 0)
 			{
 				if (errno == EINTR)
@@ -286,40 +282,21 @@ replicate_loop(int fd)
 					}
 					else
 					{
-						int recovery_status = PGRget_recovery_status();
 #ifdef PRINT_DEBUG
 						show_debug("%s:session closed",func);
 #endif
-						if (recovery_status == RECOVERY_START)
+						if (exist_replicate)
 						{
 							header_save_for_recovering.cmdSts=CMD_TYPE_OTHER;
 							header_save_for_recovering.cmdType=CMD_TYPE_CONNECTION_CLOSE;
 							header_save_for_recovering.query_size = htonl(21);
 							PGRdo_replicate(sock,&header_save_for_recovering,"PGR_CLOSE_CONNECTION");
 						}
-
-						if(exist_replicate) 
-						{
-							int recovery_status;
-
-							PGRsem_lock(RecoverySemID,SEM_NUM_OF_RECOVERY);
-							Recovery_Status_Inf->transaction_count --;
-							PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY);										
-							show_debug("transaction_status: %d",Recovery_Status_Inf->transaction_count);
-							recovery_status=PGRget_recovery_status();
-							
-							header_save_for_recovering.cmdSts=CMD_TYPE_OTHER;
-							header_save_for_recovering.cmdType=CMD_TYPE_CONNECTION_CLOSE;
-							PGRdo_replicate(sock,&header_save_for_recovering,"PGR_CLOSE_CONNECTION");
-						}
 						PGRsend_notice_quit();
 					}
 					break;
 				}
 				cnt = 0;
-				memcpy(&header_save_for_recovering,
-					   &header,
-					   sizeof(ReplicateHeader));
 				switch (header.cmdSys)
 				{
 				        case CMD_SYS_PREREPLICATE:
@@ -336,38 +313,14 @@ replicate_loop(int fd)
 				                }
 								break;
 						case CMD_SYS_REPLICATE:
+								if (exist_replicate == false)
+								{
+										exist_replicate=true;
+										memcpy(&header_save_for_recovering,
+											   &header,
+											   sizeof(ReplicateHeader));
+								}
 								status = PGRdo_replicate(sock,&header,query);
-								if (header.cmdSts == CMD_STS_TRANSACTION )
-								{
-										switch(header.cmdType)
-										{
-												case CMD_TYPE_BEGIN:
-														exist_replicate=true;
-														memcpy(&header_save_for_recovering,
-															   &header,
-															   sizeof(ReplicateHeader));
-														break;
-												case CMD_TYPE_COMMIT:
-												case CMD_TYPE_ROLLBACK:
-														exist_replicate=false;
-														break;
-										}
-								}
-								else
-								{
-										if ( header.cmdType == CMD_TYPE_COPY )
-										{
-														exist_replicate=true;
-														memcpy(&header_save_for_recovering,
-															   &header,
-															   sizeof(ReplicateHeader));
-										}
-										else if (header.cmdType == CMD_TYPE_COPY_DATA_END) 
-										{
-														exist_replicate=false;
-										}
-								}
-
 								break;
 						case CMD_SYS_LOG:
 								exist_sys_log = true;
@@ -890,11 +843,11 @@ main(int argc, char * argv[])
 	write_pid_file();
 	pgreplicate_pid = getpid();
 
-	signal(SIGINT, set_exit_processing);
-	signal(SIGQUIT, set_exit_processing);
-	signal(SIGTERM, set_exit_processing);
-	signal(SIGCHLD, child_wait);
-	signal(SIGPIPE, SIG_IGN);
+	PGRsignal(SIGINT, set_exit_processing);
+	PGRsignal(SIGQUIT, set_exit_processing);
+	PGRsignal(SIGTERM, set_exit_processing);
+	PGRsignal(SIGCHLD, child_wait);
+	PGRsignal(SIGPIPE, SIG_IGN);
 
 	if (PGRset_Conf_Data(PGR_Data_Path) != STATUS_OK)
 	{
@@ -929,12 +882,12 @@ main(int argc, char * argv[])
 	}
 	if (pid == 0)
 	{
-		signal(SIGCHLD, SIG_DFL);
-		signal(SIGHUP, quick_exit);	
-		signal(SIGINT, quick_exit);	
-		signal(SIGQUIT, quick_exit);	
-		signal(SIGTERM, quick_exit);	
-		signal(SIGPIPE, SIG_IGN);	
+		PGRsignal(SIGCHLD, SIG_DFL);
+		PGRsignal(SIGHUP, quick_exit);	
+		PGRsignal(SIGINT, quick_exit);	
+		PGRsignal(SIGQUIT, quick_exit);	
+		PGRsignal(SIGTERM, quick_exit);	
+		PGRsignal(SIGPIPE, SIG_IGN);	
 		/*
 		 * in child process,
 		 * call recovery module
