@@ -2,13 +2,13 @@
  * FILE:
  *     pgreplicate.h
  *
- * Portions Copyright (c) 2003-2005, Atsushi Mitani
+ * Portions Copyright (c) 2003-2006, Atsushi Mitani
  *--------------------------------------------------------------------
  */
 #ifndef PGREPLICATE_H
 #define PGREPLICATE_H
 
-#define PGREPLICATE_VERSION		"1.0.10a"
+#define PGREPLICATE_VERSION		"1.0.10"
 
 #include "lib/dllist.h"
 #include "../libpgc/libpgc.h"
@@ -117,6 +117,7 @@ typedef struct {
 	char dbName[DBNAME_MAX_LENGTH];
 	PGconn	* conn;
 	bool in_transaction;
+	bool exec_copy;
 }TransactionTbl;
 
 /*
@@ -295,7 +296,7 @@ extern bool	PGR_AutoCommit;
 extern unsigned int * PGR_ReplicateSerializationID;
 extern unsigned int * PGR_Send_Query_ID;
 extern unsigned int PGR_Query_ID;
-extern bool exit_processing;
+extern volatile bool exit_processing;
 extern RecoveryQueueInf RecoveryQueue;
 extern int pgreplicate_pid;
 
@@ -303,7 +304,7 @@ extern int ReplicateSock;
 
 /* smart shutdown */
 extern int Idle_Flag;
-extern bool Exit_Request;
+extern volatile bool Exit_Request;
 
 /*
  * external prototype in main.c
@@ -377,7 +378,6 @@ extern int PGRget_rlog_header(ReplicateHeader * header);
 /*
  * external prototype in cascade.c
  */
-extern void PGRinit_cascade_child() ;
 extern int PGRstartup_cascade(void);
 extern int PGRsend_lower_cascade(ReplicateHeader * header, char * query);
 extern int PGRsend_upper_cascade(ReplicateHeader * header, char * query);
@@ -390,6 +390,6 @@ extern int PGRsend_cascade(int sock , ReplicateHeader * header, char * query);
 extern int PGRcascade_main(int sock, ReplicateHeader * header, char * query);
 extern int PGRwait_notice_rlog_done(void);
 extern int PGRsend_notice_rlog_done(int sock);
-extern int PGRsend_notice_quit();
+extern int PGRsend_notice_quit(void);
 
 #endif /* PGREPLICATE_H */
