@@ -6,7 +6,7 @@
  *     This file is composed of the functions to call with the source
  *     at pgreplicate for the recovery.
  *
- * Portions Copyright (c) 2003, Atsushi Mitani
+ * Portions Copyright (c) 2003-2006, Atsushi Mitani
  *--------------------------------------------------------------------
  */
 #include "postgres.h"
@@ -74,9 +74,9 @@
  * GLOBAL VARIABLE DECLARATION
  *--------------------------------------
  */
-RecoveryPacket MasterPacketData;
-RecoveryTbl Master;
-RecoveryTbl Target;
+static RecoveryPacket MasterPacketData;
+static RecoveryTbl Master;
+static RecoveryTbl Target;
 
 
 /*--------------------------------------
@@ -101,9 +101,6 @@ static int send_vacuum(HostTbl *host, char * userName, int stage);
 static void show_recovery_packet(RecoveryPacket * packet);
 #endif			
 
-int PGRsend_load_balance_packet(RecoveryPacket * packet);
-void PGRrecovery_main(void);
-
 /*-----------------------------------------------------------
  * SYMBOL
  *    read_packet()
