@@ -6,7 +6,7 @@
  *     This file is composed of the functions to call with the source
  *     at pgreplicate for the replication.
  *
- * Portions Copyright (c) 2003-2004, Atsushi Mitani
+ * Portions Copyright (c) 2003-2006, Atsushi Mitani
  *--------------------------------------------------------------------
  */
 #include "postgres.h"
@@ -104,34 +104,8 @@ static void * thread_send_source(void * arg);
 static void * thread_send_cluster(void * arg);
 
 static int send_replicate_packet_to_server( TransactionTbl * transaction_tbl, int current_cluster, HostTbl * host_ptr, ReplicateHeader * header, char *query , char * result,unsigned int replicationId, bool recovery);
-
-int replicate_packet_send_internal(ReplicateHeader * header, char * query,int dest,int recovery_status,bool isHeldLock);
-bool PGRis_same_host(char * host1, unsigned short port1 , char * host2, unsigned short port2);
-HostTbl * PGRadd_HostTbl(HostTbl *  conf_data, int useFlag);
-HostTbl * PGRget_master(void);
-void PGRset_recovery_status(int status);
-int PGRget_recovery_status(void);
-int PGRcheck_recovered_host(void);
-int PGRset_recovered_host(HostTbl * target,int useFlag);
-int PGRinit_recovery(void);
-void PGRreplicate_exit(int exit_status);
-int PGRsend_replicate_packet_to_server( HostTbl * host_ptr, ReplicateHeader * header, char *query , char * result,unsigned int replicationId, bool recovery);
-HostTbl * PGRget_HostTbl(char * hostName,int port);
-int PGRset_queue(ReplicateHeader * header,char * query);
-int PGRset_host_status(HostTbl * host_ptr,int status);
-void PGRclear_transactions(void);
-void PGRclear_connections();
-int PGRset_replication_id(uint32_t id);
-int PGRdo_replicate(int sock,ReplicateHeader *header, char * query);
-int PGRreturn_result(int dest, char * result,int wait);
-int PGRreplicate_packet_send( ReplicateHeader * header, char * query,int dest,int recovery_status);
-char * PGRread_packet(int sock, ReplicateHeader *header);
-//void PGRnotice_replication_server(char * hostName, unsigned short portNumber,unsigned short recoveryPortNumber, unsigned short lifecheckPortNumber, char * userName);
-char * PGRread_query(int sock, ReplicateHeader *header);
-int PGRwait_transaction_count_clear(HostTbl *host);
-
-unsigned int PGRget_next_query_id(void);
-int PGRinit_transaction_table(void);
+static int check_result( PGresult * res );
+static bool compare_results(int *results, int size, int source_id);
 
 extern pthread_mutex_t transaction_table_mutex;
 
@@ -200,13 +174,13 @@ pgr_createConn( char * host, char * port,char * database, char * userName )
 		    sleep(3);
 		    show_debug("gethostbyname() failed. sleep and retrying...");
 		}
-		else if(!strncmp(PQerrorMessage(conn),"FATAL:  Sorry, too many clients already",30) ||
-			!strncmp(PQerrorMessage(conn),"FATAL:  Non-superuser connection limit",30) ) 
+		else if(!strncasecmp(PQerrorMessage(conn),"FATAL:  Sorry, too many clients already",30) ||
+			!strncasecmp(PQerrorMessage(conn),"FATAL:  Non-superuser connection limit",30) ) 
 		{
 		     sleep(3) ;
 		     show_debug("Connection overflow. sleep and retrying...");
 		}
-		else if(!strncmp(PQerrorMessage(conn),"FATAL:  The database system is starting up",40)   )
+		else if(!strncasecmp(PQerrorMessage(conn),"FATAL:  The database system is starting up",40)   )
 		{
 		  show_debug("waiting for starting up...");
 		     sleep(3) ;
@@ -366,7 +340,7 @@ getTransactionTbl( HostTbl * host_ptr, ReplicateHeader * header)
 			(transaction->hostIP == host_ip) &&
 			(transaction->port == host_ptr->port) &&
 			(transaction->srcHostIP == srcHost_ip) &&
-			(!strncmp(transaction->dbName,header->dbName,sizeof(transaction->dbName))) &&
+			(!strncasecmp(transaction->dbName,header->dbName,sizeof(transaction->dbName))) &&
 			(transaction->pid == pid))
 		{
 			pthread_mutex_unlock(&transaction_table_mutex);
@@ -743,7 +717,7 @@ set_exit_processing(int signo)
 {
 	exit_signo = signo;
 	exit_processing = true;
-	signal(signo, SIG_IGN);
+	PGRsignal(signo, SIG_IGN);
 }
 
 void
@@ -751,8 +725,15 @@ PGRreplicate_exit(int exit_status)
 {
 	char fname[256];
 	int rtn = 0;
+	sigset_t mask;
+
+	sigemptyset(&mask);
+	sigaddset(&mask, SIGTERM);
+	sigaddset(&mask, SIGINT);
+	sigaddset(&mask, SIGQUIT);
+	sigaddset(&mask, SIGCHLD);
+	sigprocmask(SIG_BLOCK, &mask, NULL);
 
-	signal(SIGCHLD, SIG_IGN);
 	kill (0, exit_signo);
 
 	child_wait(0);
@@ -959,7 +940,6 @@ write_host_status_file(HostTbl * host_ptr)
 	}
 }
 
-#if 0
 static int
 check_result( PGresult * res )
 {
@@ -973,7 +953,31 @@ check_result( PGresult * res )
 	}
 	return STATUS_OK;
 }
-#endif
+
+static bool
+compare_results(int *results, int size, int source_id)
+{
+	int i, prev = 0;
+
+	for (i = 0; i < size; i++)
+	{
+		if (i != source_id)
+		{
+			prev = results[i];
+			break;
+		}
+	}
+
+	for (; i < size; i++)
+	{
+		if (i == source_id)
+			continue;
+		if (prev != results[i])
+			return false;
+		prev = results[i];
+	}
+	return true;
+}
 
 /*--------------------------------------------------
  * SYMBOL
@@ -1269,6 +1273,7 @@ send_replicate_packet_to_server( TransactionTbl * transaction_tbl, int current_c
 	             PQclear(res);
 	
 		res = PQexec(conn, query);
+		rtn = check_result(res);
 #ifdef PRINT_DEBUG
 	show_debug("%s:PQexec send :%s",func,query);
 #endif	
@@ -1297,7 +1302,7 @@ send_replicate_packet_to_server( TransactionTbl * transaction_tbl, int current_c
 	}
 	else
 	{
-		if (!strncmp(str,PGR_LOCK_CONFLICT_NOTICE_CMD,strlen(PGR_LOCK_CONFLICT_NOTICE_CMD)))
+		if (!strncasecmp(str,PGR_LOCK_CONFLICT_NOTICE_CMD,strlen(PGR_LOCK_CONFLICT_NOTICE_CMD)))
 		{
 #ifdef PRINT_DEBUG
 			show_debug("%s:LOCK CONFLICT from PQexec",func);
@@ -1308,7 +1313,7 @@ send_replicate_packet_to_server( TransactionTbl * transaction_tbl, int current_c
 			transaction_tbl->lock = STATUS_LOCK_CONFLICT;
 			return STATUS_LOCK_CONFLICT;
 		}
-		else if (!strncmp(str,PGR_DEADLOCK_DETECT_NOTICE_CMD,strlen(PGR_DEADLOCK_DETECT_NOTICE_CMD)))
+		else if (!strncasecmp(str,PGR_DEADLOCK_DETECT_NOTICE_CMD,strlen(PGR_DEADLOCK_DETECT_NOTICE_CMD)))
 		{
 #ifdef PRINT_DEBUG
 			show_debug("%s:DEADLOCK DETECTED from PQexec",func);
@@ -1330,7 +1335,7 @@ send_replicate_packet_to_server( TransactionTbl * transaction_tbl, int current_c
 	 * if the query is end transaction process...
 	 */
 	check_delete_transaction(host_ptr,header);
-	return STATUS_OK;
+	return rtn;
 
 }
 
@@ -1359,7 +1364,6 @@ check_transaction_status(ReplicateHeader * header,
 						 HostTbl *host,
 						 TransactionTbl *transaction)
 {
-	static bool exec_copy = false;
 	if (header == (ReplicateHeader *)NULL)
 	{
 		return;
@@ -1382,14 +1386,15 @@ check_transaction_status(ReplicateHeader * header,
 	{ 
 		if ( header->cmdType == CMD_TYPE_COPY ) 
 		{
-			exec_copy = true;
-			host->transaction_count--;
+			transaction->exec_copy = true;
+			host->transaction_count++;
 		}
 		else if (header->cmdType == CMD_TYPE_COPY_DATA_END) 
 		{
-			if (exec_copy)
+			if (transaction->exec_copy)
 				host->transaction_count--;
-			exec_copy = false;						
+
+			transaction->exec_copy = false;						
 		}
 	}
 
@@ -2046,28 +2051,28 @@ read_answer(int dest)
 #endif			
 			if (answer != NULL)
 			{
-				if (!strncmp(answer,PGR_QUERY_DONE_NOTICE_CMD,strlen(PGR_QUERY_DONE_NOTICE_CMD)))
+				if (!strncasecmp(answer,PGR_QUERY_DONE_NOTICE_CMD,strlen(PGR_QUERY_DONE_NOTICE_CMD)))
 				{
 #ifdef PRINT_DEBUG
 					show_debug("%s:QUERY DONE",func);
 #endif			
 					status = STATUS_OK;
 				}
-				else if (!strncmp(answer,PGR_QUERY_ABORTED_NOTICE_CMD,strlen(PGR_QUERY_ABORTED_NOTICE_CMD)))
+				else if (!strncasecmp(answer,PGR_QUERY_ABORTED_NOTICE_CMD,strlen(PGR_QUERY_ABORTED_NOTICE_CMD)))
 				{
 #ifdef PRINT_DEBUG
 					show_debug("%s:QUERY ABORTED",func);
 #endif			
 					status = STATUS_ABORTED;
 				}
-				else if (!strncmp(answer,PGR_LOCK_CONFLICT_NOTICE_CMD,strlen(PGR_LOCK_CONFLICT_NOTICE_CMD)))
+				else if (!strncasecmp(answer,PGR_LOCK_CONFLICT_NOTICE_CMD,strlen(PGR_LOCK_CONFLICT_NOTICE_CMD)))
 				{
 #ifdef PRINT_DEBUG
 					show_debug("%s:LOCK CONFLICT !!",func);
 #endif			
 					status = STATUS_LOCK_CONFLICT;
 				}
-				else if (!strncmp(answer,PGR_DEADLOCK_DETECT_NOTICE_CMD,strlen(PGR_DEADLOCK_DETECT_NOTICE_CMD)))
+				else if (!strncasecmp(answer,PGR_DEADLOCK_DETECT_NOTICE_CMD,strlen(PGR_DEADLOCK_DETECT_NOTICE_CMD)))
 				{
 #ifdef PRINT_DEBUG
 					show_debug("%s:DEADLOCK DETECT !!",func);
@@ -2125,6 +2130,8 @@ replicate_packet_send_internal(ReplicateHeader * header, char * query,int dest,i
 	int t_cnt = 0;
 	int source_t_cnt = -1;
 
+	int *results_from_thread;
+
 
 	pthread_t thread[MAX_DB_SERVER];
 	ThreadArgInf thread_arg[MAX_DB_SERVER];
@@ -2353,22 +2360,31 @@ replicate_packet_send_internal(ReplicateHeader * header, char * query,int dest,i
 		status = STATUS_OK;
 	}    
 	pthread_attr_destroy(&attr);
+
+	results_from_thread = malloc(t_cnt * sizeof(int));
 	for ( t = 0 ; t < t_cnt; )
 	{
+		int result;
 		if (t == source_t_cnt)
 		{
 			t++;
 			continue;
 		}
-		rc = pthread_join(thread[t], (void **)NULL);
+		rc = pthread_join(thread[t], (void **)&result);
 		if ((rc != 0) && (errno == EINTR))
 		{
 			usleep(100);
 			continue;
 		}
+		results_from_thread[t] = (int)result;
 		pthread_detach(thread[t]);
 		t++;
 	}
+	
+	if (compare_results(results_from_thread, t_cnt, source_t_cnt) == false)
+		show_error("query results discrepancy between cluster servers: %s",
+				   query);
+	free(results_from_thread);
 
 	/*
 	 * send replication query to source cluster server.
@@ -2379,7 +2395,7 @@ replicate_packet_send_internal(ReplicateHeader * header, char * query,int dest,i
 	}
 	else 
 	{
-	  show_debug("thread_send_source was not executed");
+		show_debug("thread_send_source was not executed");
 	}
 	/* unset replication log */
 	if (is_need_use_rlog(header) == true)
@@ -2541,7 +2557,7 @@ thread_send_cluster(void * arg)
 	{
 		snprintf(result,PGR_MESSAGE_BUFSIZE,"%d", PGR_NOTICE_ABORT_FUNC_NO);
 #ifdef PRINT_DEBUG
-	show_debug("%s:pthread abort[%d]",func,current_cluster );
+		show_debug("%s:pthread abort[%d]",func,current_cluster );
 #endif	
 		status = PGRreturn_result(dest, result, PGR_NOWAIT_ANSWER);
 		status = STATUS_ABORTED;
@@ -2553,7 +2569,7 @@ thread_send_cluster(void * arg)
 	show_debug("%s:pthread_exit[%d]",func,current_cluster );
 #endif	
 
-	pthread_exit((void *) 0);
+	pthread_exit((void *) rtn);
 }
 
 /*--------------------------------------------------
@@ -2845,7 +2861,7 @@ get_host_ip_from_tbl(char * host)
 	while (ptr)
 	{
 		TransactionTbl *transaction = DLE_VAL(ptr);
-		if (!strncmp(transaction->host,host,sizeof(transaction->host)))
+		if (!strncasecmp(transaction->host,host,sizeof(transaction->host)))
 		{
 			pthread_mutex_unlock(&transaction_table_mutex);
 			return transaction->hostIP;
@@ -2874,7 +2890,7 @@ get_srcHost_ip_from_tbl(char * srcHost)
 	while (ptr)
 	{
 		TransactionTbl *transaction = DLE_VAL(ptr);
-		if (!strncmp(transaction->srcHost,srcHost,sizeof(transaction->srcHost)))
+		if (!strncasecmp(transaction->srcHost,srcHost,sizeof(transaction->srcHost)))
 		{
 			pthread_mutex_unlock(&transaction_table_mutex);
 
@@ -3107,7 +3123,7 @@ is_executed_query( PGconn *conn, ReplicateHeader * header)
 #endif		
 
 		if ((str != NULL) &&
-			(!strncmp(str,PGR_ALREADY_REPLICATED_NOTICE_CMD,strlen(PGR_ALREADY_REPLICATED_NOTICE_CMD))))
+			(!strncasecmp(str,PGR_ALREADY_REPLICATED_NOTICE_CMD,strlen(PGR_ALREADY_REPLICATED_NOTICE_CMD))))
 		{
 			PQclear(res);
 			return true;
