@@ -6,7 +6,7 @@
  *     This file is composed of the functions to call with the source
  *     at pgreplicate for replicate ahead log.
  *
- * Portions Copyright (c) 2004, Atsushi Mitani
+ * Portions Copyright (c) 2003-2006, Atsushi Mitani
  *--------------------------------------------------------------------
  */
 #ifdef USE_REPLICATION
@@ -53,26 +53,11 @@
 #include "replicate_com.h"
 #include "pgreplicate.h"
 
-int RLog_Recv_Sock = -1;
+static int RLog_Recv_Sock = -1;
 /*--------------------------------------
  * PROTOTYPE DECLARATION
  *--------------------------------------
  */
-int PGRwrite_rlog(ReplicateHeader * header, char * query);
-ReplicateHeader * PGRget_requested_query(ReplicateHeader * header);
-void PGRreconfirm_commit(int sock, ReplicateHeader * header);
-void PGRset_rlog(ReplicateHeader * header, char * query);
-void PGRunset_rlog(ReplicateHeader * header, char * query);
-int PGRresend_rlog_to_db(void);
-pid_t  PGR_RLog_Main(void);
-int PGRcreate_send_rlog_socket(void);
-int PGRget_sync_data(ReplicateHeader *header);
-int PGRdelete_sync_data(ReplicateHeader *header);
-int PGRsend_rlog_packet(int sock,ReplicateHeader * header, const char * query_string);
-int PGRrecv_rlog_result(int sock,void * result, int size);
-int PGRsend_rlog_to_local(ReplicateHeader * header,char * query);
-int PGRget_rlog_header(ReplicateHeader * header);
-
 static int set_query_log(ReplicateHeader * header, char * query);
 static QueryLogType * get_query_log_by_header(ReplicateHeader * header);
 static QueryLogType * get_query_log(ReplicateHeader * header);
@@ -87,6 +72,8 @@ static int recv_message(int sock,char * buf, int len);
 static int send_message(int sock, char * msg, int len);
 static void exit_rlog(int sig);
 static int reconfirm_commit(ReplicateHeader * header);
+static int PGRget_sync_data(ReplicateHeader *header);
+static int PGRdelete_sync_data(ReplicateHeader *header);
 
 
 int
@@ -665,10 +652,10 @@ PGR_RLog_Main(void)
 	{
 		return pid;
 	}
-	signal(SIGTERM,exit_rlog);
-	signal(SIGINT,exit_rlog);
-	signal(SIGQUIT,exit_rlog);
-	signal(SIGPIPE,SIG_IGN);
+	PGRsignal(SIGTERM,exit_rlog);
+	PGRsignal(SIGINT,exit_rlog);
+	PGRsignal(SIGQUIT,exit_rlog);
+	PGRsignal(SIGPIPE,SIG_IGN);
 
 	if (PGRinit_transaction_table() != STATUS_OK)
 	{
@@ -711,7 +698,12 @@ PGR_RLog_Main(void)
 		FD_ZERO(&rmask);
 		FD_SET(RLog_Recv_Sock,&rmask);
 		rtn = select(RLog_Recv_Sock+1, &rmask, (fd_set *)NULL, (fd_set *)NULL, &timeout);
-		if (rtn && FD_ISSET(RLog_Recv_Sock, &rmask))
+		if (rtn < 0)
+		{
+			if (errno == EINTR || errno == EAGAIN)
+				continue;
+		}
+		else if (rtn && FD_ISSET(RLog_Recv_Sock, &rmask))
 		{
 			Idle_Flag = BUSY_MODE ;
 			addrlen = sizeof(addr);
@@ -878,7 +870,7 @@ do_rlog(int fd)
 	return STATUS_OK;
 }
 
-int
+static int
 PGRget_sync_data(ReplicateHeader *header)
 {
 	ReplicateHeader rlog_header;
@@ -916,7 +908,7 @@ PGRget_sync_data(ReplicateHeader *header)
 
 }
 
-int
+static int
 PGRdelete_sync_data(ReplicateHeader *header)
 {
 	int status;
